// -*- C -*-

include <funcs.scad>
include <commitid.scad>

ply_th = 18;
ply_hole_dia = 15;
ply_edge_min = 10;

tile_th = 3;
post_dia = 8;

post_shorter = 1;

screw_dia = 2.2;
screw_big_dia = 3.6;
screw_big_len = 4.0;

round_edge_rad = 2.0;

round_cnr_rad = 10;

interlock_dia = 10;
interlock_fine = 0.66;

interlock_fine_slope = 1.0;
interlock_fine_lenslop = 1.0;

demo_slop = 0.1;

// cutout

machine_rear_to_front = 85;

machine_rear_profile = [
			[ -  0,  -0.00 ],
			[ -  2,  -0.10 ],
			[ -  4,  -0.20 ],
			[ -  5,  -0.30 ],
			[ -  6,  -0.40 ],
			[ -  7,  -0.50 ],
			[ -  8,  -0.75 ],
			[ - 10,  -1.50 ],
			[ - 12,  -2.25 ],
			[ - 14,  -3.25 ],
			[ - 16,  -4.25 ],
			[ - 18,  -5.75 ],
			[ - 20,  -8.00 ]
			];

machine_front_profile =  [
			  [  128,  -3.70,  ],
			  [  124,  -2.90,  ],
			  [  120,  -2.45,  ],
			  [  116,  -2.10,  ],
			  [  112,  -1.80,  ],
			  [  108,  -1.55,  ],
			  [  104,  -1.25,  ],
			  [  100,  -1.00,  ],
			  [   96,  -0.80,  ],
			  [   92,  -0.65,  ],
			  [   88,  -0.55,  ],
			  [   84,  -0.50,  ]
			  ];

cutout_l_end_y_front_slop = 0.5;
cutout_l_end_y_rear_slop = 0.5;
cutout_l_end_x_slop = 0.25;

cutout_l_end_x = 22;
cutout_l_end_y = machine_rear_to_front;
cutout_l_end_curve = 1;
cutout_l_end_y_total = cutout_l_end_y
  + cutout_l_end_y_front_slop + cutout_l_end_y_rear_slop;

tile02_tr = [-250, 0];
tile01_tr = [  0, 0];

cutout_tile01_y = 170 - 147 + cutout_l_end_y_front_slop;
cutout_tile11_x = cutout_l_end_x + cutout_l_end_curve;
cutout_tile11_y = cutout_l_end_y_total - cutout_tile01_y;

// front and rear curves

rearedge_len = 170;

rearcurve_strt_len = 52;

rearcurve_z_slop = 0.75;

rearcurve_avoid_y = 35;

rearcurve_double_inrad = 26.10 + 8.04;

reartablet_z = 2.54;
reartablet_x = 5 + 1;
reartablet_y = 8;

frontcurve_side_skew = 3.5 / 72;
frontcurve_avoid_y = 70;
frontcurve_z_slop = 0.75;

frontcurve_strt_len = 60;
frontcurve_dualcurve_angle = 90 - 65;

// calculated

TEST = false;

ply_edge_hole_dist = ply_edge_min + ply_hole_dia/2;

echo(str("HOLES IN PLY ctr dist from PLY edge = ", ply_edge_hole_dist));

hole_slop = (ply_hole_dia - post_dia)/2;
tile_hard_edge_hole_dist = ply_edge_hole_dist + hole_slop;

echo(str("HOLES IN PLY ctr dist from TILE HARD edge = ",
	 tile_hard_edge_hole_dist));

echo(str("HOLES IN PLY ctr dist from TILE ROUND edge = ",
	 tile_hard_edge_hole_dist + round_edge_rad));

thehd = [ tile_hard_edge_hole_dist, tile_hard_edge_hole_dist ];
thehd_tr = thehd;
thehd_tl = [ -thehd_tr[0], thehd_tr[1] ];
thehd_bl = -thehd_tr;
thehd_br = -thehd_tl;

tablet_z_slop = rearcurve_z_slop;

interlock_rad = interlock_dia/2;
interlock_negative_rad = interlock_rad + 0.125;

interlock_sq_adj = 0.2; // arbitrary

first_front = machine_front_profile[len(machine_front_profile)-1];

m4_define(`POST_TCROSSSZ',
          `2*( tile_hard_edge_hole_dist - test_edge + 1 )')

module Post(){
  mirror([0,0,1]) {
    difference(){
      cylinder(r= post_dia/2, h= tile_th + ply_th - post_shorter);
      translate([0,0, tile_th]) {
	cylinder(r= screw_big_dia/2, h= screw_big_len);
	cylinder(r= screw_dia/2, h= ply_th, $fn=20);
      }
    }
    if (TEST) {
      translate([0,0, tile_th/2]) {
	cube([post_dia,      POST_TCROSSSZ, tile_th], center=true);
	cube([POST_TCROSSSZ, post_dia,      tile_th], center=true);
      }
    }
  }
}

module Posts(posts) {
  for (p= posts) {
    translate(concat(p, [0]))
      Post();
  }
}

module TileBase(botleft, topright){
  size = topright - botleft;
  botleft_post = botleft + thehd_tr;
  topright_post = topright + thehd_bl;
  difference(){
    mirror([0,0,1])
      translate(concat(botleft, [0]))
      cube(concat(size, [tile_th]));
    if (!TEST) {
      cidsz = topright_post-botleft_post
	+ [-post_dia,-post_dia]
	+ [0, thehd[1]];
      cidszr = [ min(cidsz[0],50), min(cidsz[1],50) ];
      echo("CID",cidsz,cidszr);
      translate( concat(botleft_post, [ -tile_th ])
		 + 0.5 * [ post_dia, post_dia, 0 ]
		 + 0.5 * concat( cidsz - cidszr, [ 0 ]) )
	Commitid_BestCount_M(cidszr);
    }
    if (TEST) {
      crossoff = tile_hard_edge_hole_dist + POST_TCROSSSZ/2;
      cidsz = [ thehd[0], size[1] - 2*crossoff ];
      cidszr = [ cidsz[0], min(cidsz[1], 50) ];
      translate( concat(botleft + [0, crossoff] + (cidsz-cidszr)/2, [0]) )
	Commitid_BestCount(cidszr);
      difference(){
	mirror([0,0,1]) {
	  translate(concat(botleft + [test_edge,test_edge], [test_tile_th]))
	    cube(concat(size - [test_edge,test_edge]*2, [tile_th]));
	  translate(concat(botleft_post, [-1]))
	    cube(concat(topright_post-botleft_post, [tile_th+2]));
	}
	shufflesz = max(test_edge, tile_hard_edge_hole_dist)*2;
	minkowski(){
	  MachineEnvelope();
	  cube(shufflesz, center=true);
	}
      }
    }
  }
}

m4_dnl  INREFFRAME(left_cnr, right_cnr, morevars) { body; }
m4_define(`INREFFRAME',`
  length_vec = ($2) - ($1);
  length = dist2d([0,0], length_vec);
  length_uvec = length_vec / length;
  ortho_uvec = [ -length_uvec[1], length_uvec[0] ];
  m = [ [ length_uvec[0],  ortho_uvec[0], 0, ($1)[0], ],
	[ length_uvec[1],  ortho_uvec[1], 0, ($1)[1], ],
	[ 0,              0,              1,            0, ],
	[ 0,              0,              0,            1, ] ];
  $3
  multmatrix(m)
')

m4_dnl  INREFFRAME(left_cnr, right_cnr, morevars)
m4_dnl    INREFFRAME_EDGE { body; }
m4_define(`INREFFRAME_EDGE',`
  translate([0,0, -round_edge_rad])
')

module RoundEdge(left_cnr, right_cnr) {
  INREFFRAME(left_cnr, right_cnr)
    INREFFRAME_EDGE {
    difference(){
      rotate([0,90,0])
	cylinder(r= round_edge_rad, h= length, $fn=50);
      translate([-1, 0, -20])
	cube([length+2, 20, 20]);
    }
  }
}

module RoundCornerCut(ci) {
  // ci should be [this_cnr, right_cnr]
  // where right_cnr is to the right (ie, anticlockwise)
  this_cnr = ci[0];
  right_cnr = ci[1];
  offr= round_cnr_rad - round_edge_rad;
  INREFFRAME(this_cnr, right_cnr) INREFFRAME_EDGE {
    difference(){
      cube(offr*2 - 0.1, center=true);
      translate([offr, offr, 0])
	cylinder(center=true, h=20, r= offr);
    }
  }
}

module RoundCornerAdd(ci) {
  this_cnr = ci[0];
  right_cnr = ci[1];
  bigr = round_cnr_rad - round_edge_rad;
  INREFFRAME(this_cnr, right_cnr) INREFFRAME_EDGE {
    intersection(){
      cube(bigr*2 + 0.1, center=true);
      translate([bigr, bigr, 0])
	rotate_extrude(convexity=10, $fn=50)
	translate([bigr, 0])
	difference(){
	  circle(r= round_edge_rad, $fn=50);
	  mirror([1,1])
	    square([20,20]);
        }
    }
  }
}

module InterlockLobePlan(negative) {
  r = negative ? interlock_negative_rad : interlock_rad;
  ymir = negative ? 0 : 1;

  dx = sqrt(3) * r;
  $fn= 80;
  translate([thehd[0], 0]){
    mirror([0,ymir]){
      circle(r=r);
      difference(){
	translate([-dx, -0.1])
	  square([ dx*2, r/2 + 0.1 ]);
	for (xi = [-1, 1]) {
	  translate([ xi*dx, r ])
	    circle(r=r);
	}
      }
    }
  }
}

module InterlockEdgePlan(negative, nlobes, length, dosquare=true) {
  for (lobei = [ 0 : nlobes-1 ]) {
    lobex = (length - thehd[0]*2) * (lobei ? lobei / (nlobes-1) : 0);
    translate([lobex, 0, 0]) {
      InterlockLobePlan(negative);
    }
  }

  if (dosquare) {
    iadj = 0;
    slotshorter = negative ? -0.1 : interlock_fine_lenslop;
    mirror([0, negative])
      translate([slotshorter, iadj])
      square([length - slotshorter*2, interlock_fine + iadj*2]);
  }
}

module InterlockEdge(left_cnr, right_cnr, negative=0, nlobes=2) {
  plusth = negative * 1.0;
  protr = interlock_fine + interlock_sq_adj;

  z2 = -tile_th/2;
  z1 = -tile_th/2 - protr / interlock_fine_slope;
  z3 = -tile_th/2 + protr / interlock_fine_slope;

  negsign = negative ? -1 : +1;
  yprotr = negsign * protr;

  INREFFRAME(left_cnr, right_cnr) {
    for (vsect = [ // zs0            zs1      ys0,            ys1
		  [ -tile_th-plusth, plusth,  0,              0],
		  [ z1,              z2,      0, yprotr],
		  [ z2,              z3,      yprotr, 0],
		  ]) {
      zs0 = vsect[0];
      zs1 = vsect[1];
      zsd = zs1-zs0;
      ys0 = vsect[2];
      ys1 = vsect[3];
      ysd = ys1-ys0;
      sl = ysd/zsd;
      m = [ [ 1,0,   0,    0 ],
	    [ 0,1, -sl, -ys0 + negsign*interlock_sq_adj ],
	    [ 0,0,   1,  zs0 ],
	    [ 0,0,   0,    1 ] ];
      multmatrix(m)
	linear_extrude(height=zsd, convexity=10)
	InterlockEdgePlan(negative, nlobes, length, !!ysd);
    }
  }
}

function TestPiece_holes2corners(holes) =
  [ holes[0] + thehd_bl,
    holes[1] + thehd_br,
    holes[1] + thehd_tr,
    holes[0] + thehd_tl ];

module TestPiece1(){ ////toplevel
  holes = [ [-100, 0],
	    [   0, 0]
	    ];
  corners = TestPiece_holes2corners(holes);
  rcs = [corners[0], corners[1]];
  difference(){
    union(){
      TileBase(corners[0], corners[2]);
      Posts(holes);
      RoundEdge(corners[0], corners[1]);
      RoundEdge(corners[3], corners[0]);
    }
    InterlockEdge(corners[1], corners[2], 1, nlobes=1);
    RoundCornerCut(rcs);
  }
  RoundCornerAdd(rcs);
}

module TestPiece2(){ ////toplevel
  holes = [ [   0, 0],
	    [  50, 0]
	    ];
  corners = TestPiece_holes2corners(holes);
  TileBase(corners[0], corners[2]);
  Posts(holes);
  RoundEdge(corners[0], corners[1]);
  InterlockEdge(corners[3], corners[0], 0, nlobes=1);
}

module TestDemo(){ ////toplevel
  translate([ -thehd[0], 0 ])
    color("blue")
    TestPiece1();
  translate([ +thehd[0] + demo_slop, 0 ])
    TestPiece2();
}

module Machine_Arm(){
  ysz = cutout_l_end_y_total;
  // assume the round end is arc of a circle
  chordlen = dist2d([0,0], [ cutout_l_end_y, cutout_l_end_curve ]);
  endrad = cutout_l_end_y / cutout_l_end_curve * chordlen;
  
  translate([0,0,-30]) linear_extrude(height=60) {
    translate(tile01_tr + [0, (-cutout_tile01_y + cutout_tile11_y)/2]) {
      intersection(){
	translate([-50, -ysz/2])
	  square([400, ysz]);
	translate([ endrad - cutout_tile11_x - cutout_l_end_x_slop, 0 ])
	  circle(r=endrad, $fa=0.01,$fd=5);
      }
    }
  }
}

module Machine_Profile(){
  first_rear = machine_rear_profile[0];
  below_point = 0.5 * ( first_rear + first_front ) + [ 0, -10 ];
  pol = concat( machine_rear_profile,
		[ below_point ],
		machine_front_profile);
  skew_angle = atan2( first_front[1] - first_rear[1],
		      first_front[0] - first_rear[0] );
  //echo(below_point, pol, skew_angle);
  hull(){
    for (z=[0,-40]) {
      translate([0,z]) {
	//scale([1,5])
	rotate([0,0, -skew_angle ]){
	  //translate( - first_front )
	  polygon(pol);
	}
      }
    }
  }
}

module Machine_NewRearProfile(){
  // figures copied out of xfig edit boxes
  // best not to edit the posbox size if poss - just move it
  posbox = 10 * ([7.2333,-14.1267] - [-16.2289,40.0289]); // box, Green
  sideline = -10 * ([-6.2400,13.5600] - [-2.4467,28.2556]); // line, Blue
  scaleline = 10 * dist2d([-1.1911,-20.4800], [-11.2600,4.0578]); // Green2
  scaleline_mm = 12+5+10+5+3;
  sh = -[abs(posbox[0]), abs(posbox[1])];
  rot = atan2(-sideline[0], sideline[1]);
  sc = scaleline_mm / scaleline;
  //echo("SH",sh,rot,sc);
  scale(sc) rotate(rot) translate(sh){
    import("sewing-table-rear-profile.dxf", convexity=10); // spline, Pink3
  }
}

module Machine_NewRearCurve(){
  slant = atan2(4,210-10);
  //echo("SL",slant);
  translate([0,0, rearcurve_double_inrad]) rotate([slant,0,0]){
    translate([ rearcurve_double_inrad,
		0,
		-rearcurve_double_inrad + 10 ]){
      rotate([180,0,0]) rotate([0,0,90]) linear_extrude(height=30){
	hull(){
	  Machine_NewRearProfile();
	  translate([0,-100]) Machine_NewRearProfile();
	}
      }
    }
    rotate([0,90,0]) rotate([90,0,0]) {
      intersection(){
	rotate_extrude(convexity=10, $fn=64)
	  rotate(90)
	  translate([ 0, -rearcurve_double_inrad ])
	  Machine_NewRearProfile();
	translate([0,0,-500])
	  cube([500,500,1000]);
      }
    }
    translate([1,0,-rearcurve_double_inrad])
      rotate([0,-90,0]) rotate([0,0,-90])
      linear_extrude(height= rearcurve_strt_len + 1)
      Machine_NewRearProfile();
  }
}

module Machine_RearProfile(){ ////toplevel
  intersection(){
    Machine_Profile();
    translate([ -200 + machine_rear_to_front/2, -100 ]) {
      square([200,200]);
    }
  }
}

module Machine_FrontProfile(){
  intersection(){
    Machine_Profile();
    translate([ machine_rear_to_front/2, -100 ]) {
      square([200,200]);
    }
  }
}

module Machine_Curves(){ ////toplevel
  translate([ tile01_tr[0] - cutout_l_end_x + rearedge_len,
	      cutout_tile11_y,
	      0 ]){
    //%cube([20,20,20]);
    translate([ -reartablet_x,
	        -1,
	        -reartablet_z + tablet_z_slop])
      mirror([0,0,1])
      cube([ reartablet_x+1,
	     reartablet_y+1,
	     20 ]);
  }
  translate([ tile01_tr[0] - cutout_l_end_x + rearedge_len,
	      cutout_tile11_y,
	      frontcurve_z_slop ]){
    translate([ 0, -first_front[0] , 0 ])
      multmatrix([[1, -frontcurve_side_skew, 0, 0],
		  [0,  1,   0, 0],
		  [0,  0,   1, 0],
		  [0,  0,   0, 1]])
      translate([ 0, first_front[0] , 0 ])
      mirror([1,0,0]){
      rotate([0,-90,0])rotate([0,0,-90]){
	linear_extrude(height= 200)
	  Machine_FrontProfile();
      }
    }
    translate([ rearcurve_strt_len,
		0,
		rearcurve_z_slop ]){
      Machine_NewRearCurve();
    }
  }
}

module Machine(){ ////toplevel
  Machine_Arm();
  Machine_Curves();
}

module MachineEnvelope(){
  // used for testing
  p_arm_bl = [-cutout_tile11_x, -cutout_tile01_y];
  y_arm_t  = cutout_tile11_y;
  p_crv_fl = p_arm_bl + [rearedge_len, -frontcurve_avoid_y];
  y_crv_b  = y_arm_t + rearcurve_avoid_y;

  translate([0,0,-50]) linear_extrude(height= 100){
    translate(p_arm_bl) square([400, y_arm_t] - p_arm_bl);
    translate(p_crv_fl) square([400, y_crv_b] - p_crv_fl);
  }
}

function Rectangle_corners(c0, sz) =
  // returns the corners of a rectangle from c0 to c0+sz
  // if sz is positive, the corners are anticlockwise starting with c0
  [ c0 + [ 0,     0     ],
    c0 + [ sz[0], 0     ],
    c0 + [ sz[0], sz[1] ],
    c0 + [ 0,     sz[1] ] ];

function Rectangle_corners2posts(c) =
  [ c[0] + thehd_tr,
    c[1] + thehd_tl,
    c[2] + thehd_bl,
    c[3] + thehd_br ];

module Rectangle_TileBase(c) { TileBase(c[0], c[2]); }

function Posts_interpolate_one(c0,c1) = [c0, (c0+c1)/2, c1];

m4_dnl   R_EDGE(c,ix)
m4_dnl        c is from Rectangle_corners and
m4_dnl        ix is a corner number
m4_dnl    expands to two comma-separated corners:
m4_dnl    that denoted by ix, and the next one anticlockwise
m4_define(`R_EDGE',`$1[$2],$1[(($2)+1)%4]')

module FitTest_general(c0,sz, dobrace=false){
  c = Rectangle_corners(c0, sz);
  brace = [7,7,9];
  difference(){
    union(){
      Rectangle_TileBase(c);
      if (dobrace) {
	translate(concat(c0, [-brace[2] + 0.1])){
	  difference(){
	    cube(concat(sz,[brace[2]]) - [5,0,0]);
	    translate(brace + [0,0, -25])
	      cube(concat(sz, [50]) - brace*2 + [10,0,0]);
	  }
	}
      }
      RoundEdge(R_EDGE(c,1));
    }
    Machine();
  }
}

module FitTest_Entire(){ ////toplevel
  FitTest_general([-40,-80], [275,180], dobrace=true);
}

module FitTest_RearCurve(){ ////toplevel
  FitTest_general([110,0], [170,100]);
}

module Tile02(){ ////toplevel
  sz = [100,170];
  c0 = tile02_tr + -sz;
  c = Rectangle_corners(c0, sz);
  posts = Rectangle_corners2posts(c);
  rcs = [R_EDGE(c,0)];
  difference(){
    union(){
      Rectangle_TileBase(c);
      Posts(posts);
      RoundEdge(R_EDGE(c,0));
      RoundEdge(R_EDGE(c,3));
      InterlockEdge(R_EDGE(c,2), 0);
    }
    InterlockEdge(R_EDGE(c,1), 1);
    RoundCornerCut(rcs);
  }
  RoundCornerAdd(rcs);
}

module Tile12(){ ////toplevel
  sz = [100,250];
  c0 = tile02_tr + [-sz[0], 0];
  c = Rectangle_corners(c0, sz);
  posts = Rectangle_corners2posts(c);
  rcs = [R_EDGE(c,3)];
  difference(){
    union(){
      Rectangle_TileBase(c);
      Posts(posts);
      RoundEdge(R_EDGE(c,2));
      RoundEdge(R_EDGE(c,3));
    }
    InterlockEdge(R_EDGE(c,0), 1);
    InterlockEdge(R_EDGE(c,1), 1);
    RoundCornerCut(rcs);
  }
  RoundCornerAdd(rcs);
}

tile_01_11_cnr = tile01_tr + [-cutout_tile11_x, 0];
tile_11_10_cnr = tile01_tr + [0, cutout_tile11_y];
tile_01_00_cnr = tile01_tr - [0, cutout_tile01_y];

module Tile11(){ ////toplevel
  sz = [250,250];
  c0 = tile01_tr + [-sz[0],0];
  c = Rectangle_corners(c0, sz);
  cnr_posts = Rectangle_corners2posts(c);
  posts = concat(
		 Posts_interpolate_one(cnr_posts[0],
				       cnr_posts[1] - [cutout_tile11_x, 0]),
		 [ cnr_posts[1] + [0, cutout_tile11_y],
		   cnr_posts[2],
		   cnr_posts[3]
		   ]);
  difference(){
    union(){
      Rectangle_TileBase(c);
      Posts(posts);
      RoundEdge(R_EDGE(c,2));
      InterlockEdge(R_EDGE(c,3));
    }
    InterlockEdge(c[0], tile_01_11_cnr, 1);
    InterlockEdge(tile_11_10_cnr, c[2], 1);
    Machine();
  }
}    

module Tile01(){ ////toplevel
  sz = [250,170];
  c0 = tile01_tr + -sz;
  c = Rectangle_corners(c0, sz);
  cnr_posts = Rectangle_corners2posts(c);
  posts = concat(
		 Posts_interpolate_one(R_EDGE(cnr_posts,0)),
		 [ cnr_posts[2] + [0, -cutout_tile01_y] ],
		 Posts_interpolate_one(cnr_posts[2] - [cutout_tile11_x, 0],
				       cnr_posts[3])
		 );
  difference(){
    union(){
      Rectangle_TileBase(c);
      Posts(posts);
      RoundEdge(R_EDGE(c,0));
      InterlockEdge(tile_01_11_cnr, c[3]);
      InterlockEdge(R_EDGE(c,3));
    }
    InterlockEdge(c[1], tile_01_00_cnr, 1);
    Machine();
  }
}    

module Tile10(){ ////toplevel
  sz = [250,250];
  c0 = tile01_tr + [0,0];
  c = Rectangle_corners(c0, sz);
  cnr_posts = Rectangle_corners2posts(c);
  cty = cutout_tile11_y;
  rcy = cty + rearcurve_avoid_y;
  posts = [ cnr_posts[0] + [ 0,                             cty ],
	    cnr_posts[1] + [ -sz[0] + rearedge_len - cutout_l_end_x, cty ],
	    cnr_posts[1] + [ 0,                             rcy ],
	    cnr_posts[2],
	    cnr_posts[3] ];
  rcs = [R_EDGE(c,2)];
  difference(){
    union(){
      Rectangle_TileBase(c);
      Posts(posts);
      RoundEdge(R_EDGE(c,1));
      RoundEdge(R_EDGE(c,2));
      InterlockEdge(c[3], tile_11_10_cnr);
    }
    RoundCornerCut(rcs);
    Machine();
  }
  RoundCornerAdd(rcs);
}

module Tile00(){ ////toplevel
  sz = [250,170];
  c0 = tile01_tr + [0,-sz[1]];
  c = Rectangle_corners(c0, sz);

  // the edge c[1]..c[2] needs a diagonal chunk, from c1bis to c2bis
  c2bis = [ -cutout_l_end_x + rearedge_len + frontcurve_strt_len, c[2][1] ];
  c1bis = [ c[1][0],
	    c[2][1] -
	    (c[2][0] - c2bis[0]) * tan(90 - frontcurve_dualcurve_angle) ];

  cnr_posts = Rectangle_corners2posts(c);
  cty = cutout_tile01_y;
  rcy = cty + frontcurve_avoid_y;
  posts = [ cnr_posts[0],
	    cnr_posts[1],
	    cnr_posts[2] + [ 0,                             -rcy ],
	    cnr_posts[2] + [ -sz[0] + rearedge_len - cutout_l_end_x, -cty ],
	    cnr_posts[3] + [ 0,                             -cty ]
	    ];
  rcs = [R_EDGE(c,1)];
  difference(){
    union(){
      Rectangle_TileBase(c);
      Posts(posts);
      RoundEdge(R_EDGE(c,0));
      RoundEdge(R_EDGE(c,1));
      InterlockEdge(tile_01_00_cnr, c[0]);
    }
    RoundCornerCut(rcs);
    Machine();
    translate([0,0,-20]) linear_extrude(height=40) {
      polygon([ c1bis, c1bis + [50,0], c2bis + [50,0], c2bis ]);
    }
  }
  RoundCornerAdd(rcs);
}

module FitTest_FrontCurve(){ ////toplevel
  p0 = [110,-80];
  sz = [170,80];
  intersection() {
    Tile00();
    translate([0,0,-8]) linear_extrude(height=18) {
      translate(p0) square(sz);
    }
  }
}

module Demo(){ ////toplevel
  translate(demo_slop*[-2,1]) color("blue") Tile12();
  translate(demo_slop*[-2,0]) color("red")  Tile02();
  translate(demo_slop*[-2,1]) color("orange") Tile11();
  translate(demo_slop*[-2,0]) color("purple") Tile01();
  translate(demo_slop*[-3,1]) color("blue")   Tile10();
  translate(demo_slop*[-3,0]) color("red")    Tile00();
  %Machine();
}
  
//TestPiece1();
//TestPiece2();
//Demo();

//Machine_Profile();
//Machine_NewRearProfile();
//Machine_NewRearCurve();
//Machine_Curves();
//Machine();
//FitTest();
//MachineEnvelope();
